@@ -115,7 +115,8 @@ class TestQuery(unittest.TestCase):
 
         self.mock_job_collection.query.assert_called_once_with(
             projectId=self.project_id,
-            body={'query': self.query, 'timeoutMs': 10000}
+            body={'query': self.query, 'timeoutMs': 10000, 'dryRun': False,
+                  'maxResults': None}
         )
         self.assertEquals(job_id, 'spiderman')
         self.assertEquals(results, [])
@@ -142,7 +143,7 @@ class TestQuery(unittest.TestCase):
         self.mock_job_collection.query.assert_called_once_with(
             projectId=self.project_id,
             body={'query': self.query, 'timeoutMs': 10000,
-                  'maxResults': max_results}
+                  'maxResults': max_results, 'dryRun': False}
         )
         self.assertEquals(job_id, 'spiderman')
         self.assertEquals(results, [])
@@ -167,11 +168,58 @@ class TestQuery(unittest.TestCase):
 
         self.mock_job_collection.query.assert_called_once_with(
             projectId=self.project_id,
-            body={'query': self.query, 'timeoutMs': timeout * 1000}
+            body={'query': self.query, 'timeoutMs': timeout * 1000, 'dryRun':
+                  False, 'maxResults': None}
         )
         self.assertEquals(job_id, 'spiderman')
         self.assertEquals(results, [])
 
+    def test_query_dry_run_valid(self):
+        """Ensure that None and an empty list is returned from the query when
+        dry_run is True and the query is valid.
+        """
+
+        mock_query_job = mock.Mock()
+
+        mock_query_job.execute.return_value = {'jobReference': {}}
+
+        self.mock_job_collection.query.return_value = mock_query_job
+
+        job_id, results = self.client.query(self.query, dry_run=True)
+
+        self.mock_job_collection.query.assert_called_once_with(
+            projectId=self.project_id,
+            body={'query': self.query, 'timeoutMs': 10000, 'maxResults': None,
+                  'dryRun': True}
+        )
+        self.assertIsNone(job_id)
+        self.assertEqual([], results)
+
+    def test_query_dry_run_invalid(self):
+        """Ensure that None and a dict is returned from the query when dry_run
+        is True and the query is invalid.
+        """
+        from apiclient.errors import HttpError
+
+        mock_query_job = mock.Mock()
+
+        mock_query_job.execute.side_effect = HttpError(
+            'crap', '{"message": "Bad query"}')
+
+        self.mock_job_collection.query.return_value = mock_query_job
+
+        job_id, results = self.client.query('%s blah' % self.query,
+                                            dry_run=True)
+
+        self.mock_job_collection.query.assert_called_once_with(
+            projectId=self.project_id,
+            body={'query': '%s blah' % self.query, 'timeoutMs': 10000,
+                  'maxResults': None,
+                  'dryRun': True}
+        )
+        self.assertIsNone(job_id)
+        self.assertEqual({'message': 'Bad query'}, results)
+
     def test_query_with_results(self):
         """Ensure that we retrieve the job id from the query and results if
         they are available.
@@ -193,7 +241,8 @@ class TestQuery(unittest.TestCase):
 
         self.mock_job_collection.query.assert_called_once_with(
             projectId=self.project_id,
-            body={'query': self.query, 'timeoutMs': 10000}
+            body={'query': self.query, 'timeoutMs': 10000, 'dryRun': False,
+                  'maxResults': None}
         )
         self.assertEquals(job_id, 'spiderman')
         self.assertEquals(results, [{'foo': 10}])
@@ -515,6 +564,49 @@ class TestGetQuerySchema(unittest.TestCase):
         self.assertEquals(result_schema, [])
 
 
+class TestGetTableSchema(unittest.TestCase):
+
+    def setUp(self):
+        self.mock_bq_service = mock.Mock()
+        self.mock_tables = mock.Mock()
+        self.mock_bq_service.tables.return_value = self.mock_tables
+        self.table = 'table'
+        self.project = 'project'
+        self.dataset = 'dataset'
+        self.client = client.BigQueryClient(self.mock_bq_service, self.project)
+
+    def test_table_exists(self):
+        """Ensure that the table schema is returned if the table exists."""
+
+        expected = [
+            {'type': 'FLOAT', 'name': 'foo', 'mode': 'NULLABLE'},
+            {'type': 'INTEGER', 'name': 'bar', 'mode': 'NULLABLE'},
+            {'type': 'INTEGER', 'name': 'baz', 'mode': 'NULLABLE'},
+        ]
+
+        self.mock_tables.get.return_value.execute.return_value = \
+            {'schema': {'fields': expected}}
+
+        self.assertEqual(
+            expected, self.client.get_table_schema(self.dataset, self.table))
+        self.mock_tables.get.assert_called_once_with(
+            projectId=self.project, tableId=self.table, datasetId=self.dataset)
+        self.mock_tables.get.return_value.execute.assert_called_once_with()
+
+    def test_table_does_not_exist(self):
+        """Ensure that None is returned if the table doesn't exist."""
+        from apiclient.errors import HttpError
+
+        self.mock_tables.get.return_value.execute.side_effect = \
+            HttpError({'status': "404"}, '{}')
+
+        self.assertIsNone(
+            self.client.get_table_schema(self.dataset, self.table))
+        self.mock_tables.get.assert_called_once_with(
+            projectId=self.project, tableId=self.table, datasetId=self.dataset)
+        self.mock_tables.get.return_value.execute.assert_called_once_with()
+
+
 @mock.patch('bigquery.client.BigQueryClient._get_query_results')
 class TestGetQueryRows(unittest.TestCase):
 
